@@ -20,16 +20,13 @@
 from __future__ import absolute_import
 
 import logging
-from builtins import open
 
 import pytest
-import ruamel.yaml as yaml
-from importlib_resources import open_text
 
-import memote.suite.templates as templates
 from memote.suite import TEST_DIRECTORY
 from memote.suite.collect import ResultCollectionPlugin
-from memote.suite.reporting import SnapshotReport, HistoryReport
+from memote.suite.reporting import (
+    SnapshotReport, DiffReport, HistoryReport, ReportConfiguration)
 
 __all__ = ("test_model", "snapshot_report", "diff_report", "history_report")
 
@@ -37,7 +34,7 @@ LOGGER = logging.getLogger(__name__)
 
 
 def test_model(model, results=False, pytest_args=None,
-               exclusive=None, skip=None):
+               exclusive=None, skip=None, experimental=None):
     """
     Test a model and optionally store results as JSON.
 
@@ -69,7 +66,8 @@ def test_model(model, results=False, pytest_args=None,
         pytest_args.extend(["--tb", "short"])
     if TEST_DIRECTORY not in pytest_args:
         pytest_args.append(TEST_DIRECTORY)
-    plugin = ResultCollectionPlugin(model, exclusive=exclusive, skip=skip)
+    plugin = ResultCollectionPlugin(model, exclusive=exclusive, skip=skip,
+                                    experimental_config=experimental)
     code = pytest.main(pytest_args, plugins=[plugin])
     if results:
         return code, plugin.results
@@ -77,52 +75,70 @@ def test_model(model, results=False, pytest_args=None,
         return code
 
 
-def snapshot_report(result, config=None, filename=None):
+def snapshot_report(result, config=None, html=True):
     """
-    Test a model and save a basic report.
+    Generate a snapshot report from a result set and configuration.
 
     Parameters
     ----------
     result : memote.MemoteResult
         Nested dictionary structure as returned from the test suite.
     config : dict, optional
-        The final test report configuration.
-    filename : str or pathlib.Path
-        A filename for the HTML report.
+        The final test report configuration (default None).
+    html : bool, optional
+        Whether to render the report as full HTML or JSON (default True).
 
     """
     if config is None:
-        with open_text(templates, "test_config.yml") as file_handle:
-            LOGGER.debug("Loading default snapshot configuration.")
-            config = yaml.load(file_handle)
+        config = ReportConfiguration.load()
     report = SnapshotReport(result=result, configuration=config)
-    LOGGER.info("Writing snapshot report to '%s'.", filename)
-    with open(filename, "w", encoding="utf-8") as file_h:
-        file_h.write(report.render_html())
+    if html:
+        return report.render_html()
+    else:
+        return report.render_json()
 
 
-def history_report(repository, manager, filename, index="hash"):
+def history_report(history, config=None, html=True):
     """
     Test a model and save a history report.
 
     Parameters
     ----------
-    repository : git.Repo, optional
-        An instance of the working directory git repository.
-    manager : memote.RepoResultManager
+    history : memote.HistoryManager
         The manager grants access to previous results.
-    filename : str or pathlib.Path
-        A filename for the HTML report.
-    index : {"hash", "time"}, optional
-        The default horizontal axis type for all plots.
+    config : dict, optional
+        The final test report configuration.
+    html : bool, optional
+        Whether to render the report as full HTML or JSON (default True).
 
     """
-    report = HistoryReport(repository, manager, index=index)
-    LOGGER.info("Writing history report '%s'.", filename)
-    with open(filename, "w", encoding="utf-8") as file_h:
-        file_h.write(report.render_html())
+    if config is None:
+        config = ReportConfiguration.load()
+    report = HistoryReport(history=history, configuration=config)
+    if html:
+        return report.render_html()
+    else:
+        return report.render_json()
 
 
-def diff_report():
-    u"""Coming soon™."""
-    raise NotImplementedError(u"Coming soon™.")
+def diff_report(diff_results, config=None, html=True):
+    """
+    Generate a diff report from a result set and configuration.
+
+    Parameters
+    ----------
+    result : memote.MemoteResult
+        Nested dictionary structure as returned from the test suite.
+    config : dict, optional
+        The final test report configuration (default None).
+    html : bool, optional
+        Whether to render the report as full HTML or JSON (default True).
+
+    """
+    if config is None:
+        config = ReportConfiguration.load()
+    report = DiffReport(diff_results=diff_results, configuration=config)
+    if html:
+        return report.render_html()
+    else:
+        return report.render_json()
