@@ -21,33 +21,37 @@ from __future__ import absolute_import, division
 
 import memote.support.basic as basic
 import memote.support.helpers as helpers
-from memote.utils import annotate, get_ids, truncate, wrapper
+from memote.utils import (
+    annotate, get_ids, get_ids_and_bounds, truncate, wrapper)
 
 
-@annotate(title="Model Identifier", type="raw")
+@annotate(title="Model Identifier", format_type="raw")
 def test_model_id_presence(read_only_model):
     """
     Expect that the model has an identifier.
 
-    The MIRIAM guidelines require a model to be named. While it is not
-    required, the ID will be displayed on the memote reports, which helps to
-    distinguish the output clearly.
+    The MIRIAM guidelines require a model to be identified via an ID.
+    Further, the ID will be displayed on the memote snapshot
+    report, which helps to distinguish the output clearly.
+
     """
     ann = test_model_id_presence.annotation
     assert hasattr(read_only_model, "id")
     ann["data"] = read_only_model.id
+    ann["message"] = "The model ID is {}".format(ann["data"])
     assert bool(read_only_model.id)
 
 
-@annotate(title="Total Number of Genes", type="count")
+@annotate(title="Total Genes", format_type="count")
 def test_genes_presence(read_only_model):
     """
-    Expect that more than one gene is defined in the model.
+    Expect that at least one gene is defined in the model.
 
     A metabolic model can still be a useful tool without any
     genes, however there are certain methods which rely on the presence of
     genes and, more importantly, the corresponding gene-protein-reaction
     rules. This test requires that there is at least one gene defined.
+
     """
     ann = test_genes_presence.annotation
     assert hasattr(read_only_model, "genes")
@@ -57,13 +61,13 @@ def test_genes_presence(read_only_model):
     assert len(ann["data"]) >= 1, ann["message"]
 
 
-@annotate(title="Total Number of Reactions", type="count")
+@annotate(title="Total Reactions", format_type="count")
 def test_reactions_presence(read_only_model):
     """
-    Expect that more than one reaction is defined in the model.
+    Expect that at least one reaction is defined in the model.
 
     To be useful a metabolic model should consist at least of a few reactions.
-    This test simply checks if there are more than one.
+    This test simply checks if there are more than zero reactions.
     """
     ann = test_reactions_presence.annotation
     assert hasattr(read_only_model, "reactions")
@@ -73,14 +77,14 @@ def test_reactions_presence(read_only_model):
     assert len(ann["data"]) >= 1, ann["message"]
 
 
-@annotate(title="Total Number of Metabolites", type="count")
+@annotate(title="Total Metabolites", format_type="count")
 def test_metabolites_presence(read_only_model):
     """
-    Expect that more than one metabolite is defined in the model.
+    Expect that at least one metabolite is defined in the model.
 
     To be useful a metabolic model should consist at least of a few
     metabolites that are converted by reactions.
-    This test simply checks if there are more than one metabolites defined.
+    This test simply checks if there are more than zero metabolites.
     """
     ann = test_metabolites_presence.annotation
     assert hasattr(read_only_model, "metabolites")
@@ -90,40 +94,17 @@ def test_metabolites_presence(read_only_model):
     assert len(ann["data"]) >= 1, ann["message"]
 
 
-@annotate(title="Total Number of Transport Reactions", type="count")
-def test_transport_reaction_presence(read_only_model):
-    """
-    Expect more than one transport reaction to be defined in the model.
-
-    Cellular metabolism in any organism usually involves the transport of
-    metabolites across a lipid bi-layer. Hence, this test checks that there is
-    at least one reaction, which transports metabolites from one compartment
-    to another.
-
-    A transport reaction is defined as follows:
-    1. It contains metabolites from at least two compartments and
-    2. at least one metabolite undergoes no chemical conversion, i.e.,
-    the formula stays the same on both sides of the equation.
-
-    This test will not be able to identify transport via the PTS System.
-    """
-    ann = test_transport_reaction_presence.annotation
-    ann["data"] = get_ids(helpers.find_transport_reactions(read_only_model))
-    ann["message"] = wrapper.fill(
-        """A total of {:d} ({:.2%}) transport reactions are defined in the
-        model, this excludes purely metabolic reactions, exchanges, or
-        pseudo-reactions: {}""".format(
-            len(ann["data"]), ann["metric"], truncate(ann["data"])))
-    assert len(ann["data"]) >= 1, ann["message"]
-
-
-@annotate(title="Metabolites without Formula", type="count")
+@annotate(title="Metabolites without Formula", format_type="count")
 def test_metabolites_formula_presence(read_only_model):
     """
     Expect all metabolites to have a formula.
 
     To be able to ensure that reactions are mass-balanced, all model
-    metabolites ought to be provided with a chemical formula.
+    metabolites ought to be provided with a chemical formula. Since it may be
+    difficult to obtain formulas for certain metabolites this test serves as a
+    mere report. Models can still be stoichiometrically consistent even
+    when chemical formulas are not defined for each metabolite.
+
     """
     ann = test_metabolites_formula_presence.annotation
     ann["data"] = get_ids(
@@ -136,13 +117,16 @@ def test_metabolites_formula_presence(read_only_model):
     assert len(ann["data"]) == 0, ann["message"]
 
 
-@annotate(title="Metabolites without Charge", type="count")
+@annotate(title="Metabolites without Charge", format_type="count")
 def test_metabolites_charge_presence(read_only_model):
     """
     Expect all metabolites to have charge information.
 
     To be able to ensure that reactions are charge-balanced, all model
-    metabolites ought to be provided with a charge.
+    metabolites ought to be provided with a charge. Since it may be
+    difficult to obtain charges for certain metabolites this test serves as a
+    mere report. Models can still be stoichiometrically consistent even
+    when charge information is not defined for each metabolite.
     """
     ann = test_metabolites_charge_presence.annotation
     ann["data"] = get_ids(
@@ -155,7 +139,7 @@ def test_metabolites_charge_presence(read_only_model):
     assert len(ann["data"]) == 0, ann["message"]
 
 
-@annotate(title="Reactions without GPR", type="count")
+@annotate(title="Reactions without GPR", format_type="count")
 def test_gene_protein_reaction_rule_presence(read_only_model):
     """
     Expect all non-exchange reactions to have a GPR rule.
@@ -179,7 +163,8 @@ def test_gene_protein_reaction_rule_presence(read_only_model):
     assert len(ann["data"]) == 0, ann["message"]
 
 
-@annotate(title="Non-Growth Associated Maintenance Reaction", type="count")
+@annotate(title="Non-Growth Associated Maintenance Reaction",
+          format_type="count")
 def test_ngam_presence(read_only_model):
     """
     Expect a single non growth-associated maintenance reaction.
@@ -187,7 +172,10 @@ def test_ngam_presence(read_only_model):
     The Non-Growth Associated Maintenance reaction (NGAM) is an
     ATP-hydrolysis reaction added to metabolic models to represent energy
     expenses that the cell invests in continuous processes independent of
-    the growth rate.
+    the growth rate. Memote tries to infer this reaction from a list of
+    buzzwords, and the stoichiometry and components of a simple ATP-hydrolysis
+    reaction. Please see the API documentation for ``basic.find_ngam`` for a
+    more detailed explanation.
     """
     ann = test_ngam_presence.annotation
     ann["data"] = get_ids(basic.find_ngam(read_only_model))
@@ -197,7 +185,7 @@ def test_ngam_presence(read_only_model):
     assert len(ann["data"]) == 1, ann["message"]
 
 
-@annotate(title="Metabolic Coverage", type="percent")
+@annotate(title="Metabolic Coverage", format_type="percent")
 def test_metabolic_coverage(read_only_model):
     u"""
     Expect a model to have a metabolic coverage >= 1.
@@ -218,7 +206,7 @@ def test_metabolic_coverage(read_only_model):
     assert ann["metric"] >= 1, ann["message"]
 
 
-@annotate(title="Total Number of Compartments", type="count")
+@annotate(title="Total Compartments", format_type="count")
 def test_compartments_presence(read_only_model):
     """
     Expect that more than two compartments are defined in the model.
@@ -242,7 +230,7 @@ def test_compartments_presence(read_only_model):
     assert len(ann["data"]) >= 3, ann["message"]
 
 
-@annotate(title="Number of Enzyme Complexes", type="count")
+@annotate(title="Enzyme Complexes", format_type="count")
 def test_protein_complex_presence(read_only_model):
     """
     Expect that more than one enzyme complex is present in the model.
@@ -264,7 +252,7 @@ def test_protein_complex_presence(read_only_model):
     assert len(ann["data"]) >= 1, ann["message"]
 
 
-@annotate(title="Number of Purely Metabolic Reactions", type="count")
+@annotate(title="Purely Metabolic Reactions", format_type="count")
 def test_find_pure_metabolic_reactions(read_only_model):
     """
     Expect at least one pure metabolic reaction to be defined in the model.
@@ -287,9 +275,66 @@ def test_find_pure_metabolic_reactions(read_only_model):
     assert len(ann["data"]) >= 1, ann["message"]
 
 
-@annotate(title="Number of Transport Reactions", type="count")
+@annotate(title="Purely Metabolic Reactions with Constraints",
+          format_type="count")
+def test_find_constrained_pure_metabolic_reactions(read_only_model):
+    """
+    Expect zero or more purely metabolic reactions to have fixed constraints.
+
+    If a reaction is neither a transport reaction, a biomass reaction nor a
+    boundary reaction, it is counted as a purely metabolic reaction. This test
+    requires the presence of metabolite formula to be able to identify
+    transport reactions. This test simply reports the number of purely
+    metabolic reactions that have fixed constraints and does not have any
+    mandatory 'pass' criteria.
+    """
+    ann = test_find_constrained_pure_metabolic_reactions.annotation
+    pmr = basic.find_pure_metabolic_reactions(read_only_model)
+    ann["data"] = get_ids_and_bounds(
+        [rxn for rxn in pmr if basic.is_constrained_reaction(
+            read_only_model, rxn)])
+    ann["metric"] = len(ann["data"]) / len(pmr)
+    ann["message"] = wrapper.fill(
+        """A total of {:d} ({:.2%}) purely metabolic reactions have fixed
+        constraints in the model, this excludes transporters, exchanges, or
+        pseudo-reactions: {}""".format(len(ann["data"]), ann["metric"],
+                                       truncate(ann["data"])))
+
+
+@annotate(title="Transport Reactions", format_type="count")
 def test_find_transport_reactions(read_only_model):
-    """Expect >= 1 transport reactions are present in the read_only_model."""
+    """
+    Expect >= 1 transport reactions are present in the read_only_model.
+
+    Cellular metabolism in any organism usually involves the transport of
+    metabolites across a lipid bi-layer. This test reports how many
+    of these reactions, which transports metabolites from one compartment
+    to another, are present in the model, as at least one transport reaction
+    must be present for cells to take up nutrients and/or excrete waste.
+
+    A transport reaction is defined as follows:
+    1. It contains metabolites from at least 2 compartments and
+    2. at least 1 metabolite undergoes no chemical reaction, i.e.,
+    the formula and/or annotation stays the same on both sides of the equation.
+
+    A notable exception is transport via PTS, which also contains the following
+    restriction:
+    3. The transported metabolite(s) are transported into a compartment through
+    the exchange of a phosphate.
+
+    An example of transport via PTS would be
+    pep(c) + glucose(e) -> glucose-6-phosphate(c) + pyr(c)
+
+    Reactions similar to transport via PTS (referred to as "modified transport
+    reactions") follow a similar pattern:
+    A(x) + B-R(y) -> A-R(y) + B(y)
+
+    Such modified transport reactions can be detected, but only when the
+    formula is defined for all metabolites in a particular reaction. If this
+    is not the case, transport reactions are identified through annotations,
+    which cannot detect modified transport reactions.
+
+    """
     ann = test_find_transport_reactions.annotation
     ann["data"] = get_ids(helpers.find_transport_reactions(read_only_model))
     ann["metric"] = len(ann["data"]) / len(read_only_model.reactions)
@@ -301,18 +346,67 @@ def test_find_transport_reactions(read_only_model):
     assert len(ann["data"]) >= 1, ann["message"]
 
 
-@annotate(title="Fraction of Transport Reactions without GPR", type="percent")
+@annotate(title="Transport Reactions with Constraints",
+          format_type="count")
+def test_find_constrained_transport_reactions(read_only_model):
+    """
+    Expect zero or more transport reactions to have fixed constraints.
+
+    Cellular metabolism in any organism usually involves the transport of
+    metabolites across a lipid bi-layer. Hence, this test reports how many
+    of these reactions, which transports metabolites from one compartment
+    to another, have fixed constraints. This test does not have any mandatory
+    'pass' criteria.
+
+    A transport reaction is defined as follows:
+    1. It contains metabolites from at least 2 compartments and
+    2. at least 1 metabolite undergoes no chemical reaction, i.e.,
+    the formula and/or annotation stays the same on both sides of the equation.
+
+    A notable exception is transport via PTS, which also contains the following
+    restriction:
+    3. The transported metabolite(s) are transported into a compartment through
+    the exchange of a phosphate.
+
+    An example of transport via PTS would be
+    pep(c) + glucose(e) -> glucose-6-phosphate(c) + pyr(c)
+
+    Reactions similar to transport via PTS (referred to as "modified transport
+    reactions") follow a similar pattern:
+    A(x) + B-R(y) -> A-R(y) + B(y)
+
+    Such modified transport reactions can be detected, but only when a formula
+    field exists for all metabolites in a particular reaction. If this is not
+    the case, transport reactions are identified through annotations, which
+    cannot detect modified transport reactions.
+
+    """
+    ann = test_find_constrained_transport_reactions.annotation
+    transporters = helpers.find_transport_reactions(read_only_model)
+    ann["data"] = get_ids_and_bounds(
+        [rxn for rxn in transporters if basic.is_constrained_reaction(
+            read_only_model, rxn)])
+    ann["metric"] = len(ann["data"]) / len(transporters)
+    ann["message"] = wrapper.fill(
+        """A total of {:d} ({:.2%}) transport reactions have fixed
+        constraints in the model: {}""".format(len(ann["data"]), ann["metric"],
+                                               truncate(ann["data"])))
+
+
+@annotate(title="Fraction of Transport Reactions without GPR",
+          format_type="percent")
 def test_transport_reaction_gpr_presence(read_only_model):
     """
     Expect a small fraction of transport reactions not to have a GPR rule.
 
     As it is hard to identify the exact transport processes within a cell,
     transport reactions are often added purely for modeling purposes.
-    Highlighting where assumptions have been made vs where
+    Highlighting where assumptions have been made versus where
     there is proof may help direct the efforts to improve transport and
     transport energetics of the tested metabolic model.
     However, transport reactions without GPR may also be valid:
     Diffusion, or known reactions with yet undiscovered genes likely lack GPR.
+
     """
     # TODO: Update threshold with improved insight from meta study.
     ann = test_transport_reaction_gpr_presence.annotation
@@ -329,7 +423,29 @@ def test_transport_reaction_gpr_presence(read_only_model):
     assert ann["metric"] < 0.2, ann["message"]
 
 
-@annotate(title="Number of Unique Metabolites", type="count")
+@annotate(title="Number of Reversible Oxygen-Containing Reactions",
+          format_type="count")
+def test_find_reversible_oxygen_reactions(read_only_model):
+    """
+    Expect zero or more oxygen-containing reactions to be reversible.
+
+    The directionality of oxygen-producing/-consuming reactions affects the
+    model's ability to grow anaerobically i.e. create faux-anaerobic organisms.
+    This test reports how many of these oxygen-containing reactions are
+    reversible. This test does not have any mandatory 'pass' criteria.
+
+    """
+    ann = test_find_reversible_oxygen_reactions.annotation
+    o2_rxns = basic.find_oxygen_reactions(read_only_model)
+    ann["data"] = get_ids([rxn for rxn in o2_rxns if rxn.reversibility])
+    ann["metric"] = len(ann["data"]) / len(o2_rxns)
+    ann["message"] = wrapper.fill(
+        """There are a total of {} reversible oxygen-containing reactions
+        ({:.2%} of all oxygen-containing reactions): {}""".format(
+            len(ann["data"]), ann["metric"], truncate(ann["data"])))
+
+
+@annotate(title="Unique Metabolites", format_type="count")
 def test_find_unique_metabolites(read_only_model):
     """
     Expect there to be less metabolites when removing compartment tag.
@@ -350,3 +466,62 @@ def test_find_unique_metabolites(read_only_model):
         total of {} ({:.2%}) unique metabolites in the model: {}""".format(
             len(ann["data"]), ann["metric"], truncate(ann["data"])))
     assert len(ann["data"]) < len(read_only_model.metabolites), ann["message"]
+
+
+@annotate(title="Duplicate Metabolites in Identical Compartments",
+          format_type="count")
+def test_find_duplicate_metabolites_in_compartments(read_only_model):
+    """
+    Expect there to be zero duplicate metabolites in the same compartments.
+
+    The main reason for having this test is to help cleaning up merged models
+    or models from automated reconstruction pipelines as these are prone to
+    having identical metabolites from different namespaces
+    (hence different IDs). This test therefore expects that every metabolite
+    in any particular compartment has unique inchikey values.
+
+    """
+    ann = test_find_duplicate_metabolites_in_compartments.annotation
+    ann["data"] = basic.find_duplicate_metabolites_in_compartments(
+        read_only_model)
+    ann["message"] = wrapper.fill(
+        """There are a total of {} metabolites in the model which
+        have duplicates in the same compartment: {}""".format(
+            len(ann["data"]), truncate(ann["data"])))
+    assert len(ann["data"]) == 0, ann["message"]
+
+
+@annotate(title="Duplicate Reactions", format_type="count")
+def test_find_duplicate_reactions(read_only_model):
+    """
+    Expect there to be zero duplicate reactions.
+
+    The main reason for having this test is to help cleaning up merged models
+    or models from automated reconstruction pipelines as these are prone to
+    having identical reactions from different namespaces (hence different IDs).
+    This test therefore expects that every reaction has unique identifier
+    values (i.e. unique BRENDA, BiGG, KEGG, etc. values).
+
+    """
+    ann = test_find_duplicate_reactions.annotation
+    ann["data"] = basic.find_duplicate_reactions(read_only_model)
+    ann["message"] = wrapper.fill(
+        """There are a total of {} reactions in the model which
+        have duplicates: {}""".format(len(ann["data"]), truncate(ann["data"])))
+    assert len(ann["data"]) == 0, ann["message"]
+
+
+@annotate(title="Medium Components", format_type="count")
+def test_find_medium_metabolites(read_only_model):
+    """
+    Expect zero or more metabolites to be set as medium.
+
+    This test checks all boundary reactions in the model that permit flux
+    towards creating a metabolite, and reports those metabolites. This test
+    does not have any mandatory 'pass' criteria.
+    """
+    ann = test_find_medium_metabolites.annotation
+    ann["data"] = basic.find_medium_metabolites(read_only_model)
+    ann["message"] = wrapper.fill(
+        """There are a total of {} metabolites in the currently set medium
+        in the model: {}""".format(len(ann["data"]), truncate(ann["data"])))
