@@ -23,12 +23,12 @@ In the storage of test results we rely on the order of the biomass fixtures
 to remain the same as the parametrized test cases.
 """
 
-from __future__ import absolute_import
+from __future__ import absolute_import, division
 
 import logging
 
 import pytest
-import numpy as np
+from cobra.exceptions import OptimizationError
 
 import memote.support.biomass as biomass
 import memote.support.helpers as helpers
@@ -39,7 +39,7 @@ LOGGER = logging.getLogger(__name__)
 BIOMASS_IDS = pytest.memote.biomass_ids
 
 
-@annotate(title="Amount of Biomass Reactions", type="count")
+@annotate(title="Biomass Reactions Identified", format_type="count")
 def test_biomass_presence():
     """
     Expect the model to contain at least one biomass reaction.
@@ -65,13 +65,16 @@ def test_biomass_presence():
     assert len(ann["data"]) > 0, ann["message"]
 
 
+# TODO: Mark this test as skipped if the biomass components lack formulas!
 @pytest.mark.parametrize("reaction_id", BIOMASS_IDS)
-@annotate(title="Biomass Consistency", type="number", data=dict(),
+@annotate(title="Biomass Consistency", format_type="number", data=dict(),
           message=dict(), metric=dict())
 def test_biomass_consistency(read_only_model, reaction_id):
     """
     Expect biomass components to sum up to 1 g[CDW].
 
+    This test only yields sensible results if all biomass precursor
+    metabolites have chemical formulas assigned to them.
     The molecular weight of the biomass reaction in metabolic models is
     defined to be equal to 1 g/mmol. Conforming to this is essential in order
     to be able to reliably calculate growth yields, to cross-compare models,
@@ -86,12 +89,14 @@ def test_biomass_consistency(read_only_model, reaction_id):
         """The component molar mass of the biomass reaction {} sums up to {}
         which is outside of the 1e-03 margin from 1 mmol / g[CDW] / h.
         """.format(reaction_id, ann["data"][reaction_id]))
-    assert np.isclose(
-        ann["data"][reaction_id], 1.0, atol=1e-03), ann["message"][reaction_id]
+    # To account for numerical innacuracies, a range from 1-1e0-3 to 1+1e-06
+    # is implemented in the assertion check
+    assert (1 - 1e-03) < ann["data"][reaction_id] < (1 + 1e-06), \
+        ann["message"][reaction_id]
 
 
 @pytest.mark.parametrize("reaction_id", BIOMASS_IDS)
-@annotate(title="Biomass Production At Default State", type="number",
+@annotate(title="Biomass Production In Default Medium", format_type="number",
           data=dict(), message=dict(), metric=dict())
 def test_biomass_default_production(model, reaction_id):
     """
@@ -113,7 +118,7 @@ def test_biomass_default_production(model, reaction_id):
 
 
 @pytest.mark.parametrize("reaction_id", BIOMASS_IDS)
-@annotate(title="Biomass Production In Complete Medium", type="number",
+@annotate(title="Biomass Production In Complete Medium", format_type="number",
           data=dict(), message=dict(), metric=dict())
 def test_biomass_open_production(model, reaction_id):
     """
@@ -136,8 +141,8 @@ def test_biomass_open_production(model, reaction_id):
 
 
 @pytest.mark.parametrize("reaction_id", BIOMASS_IDS)
-@annotate(title="Blocked Biomass Precursors At Default State", type="count",
-          data=dict(), metric=dict(), message=dict())
+@annotate(title="Blocked Biomass Precursors In Default Medium",
+          format_type="count", data=dict(), metric=dict(), message=dict())
 def test_biomass_precursors_default_production(read_only_model, reaction_id):
     """
     Expect production of all biomass precursors in default medium.
@@ -169,8 +174,8 @@ def test_biomass_precursors_default_production(read_only_model, reaction_id):
 
 
 @pytest.mark.parametrize("reaction_id", BIOMASS_IDS)
-@annotate(title="Blocked Biomass Precursors In Complete Medium", type="count",
-          data=dict(), metric=dict(), message=dict())
+@annotate(title="Blocked Biomass Precursors In Complete Medium",
+          format_type="count", data=dict(), metric=dict(), message=dict())
 def test_biomass_precursors_open_production(model, reaction_id):
     """
     Expect precursor production in complete medium.
@@ -204,10 +209,10 @@ def test_biomass_precursors_open_production(model, reaction_id):
 
 @pytest.mark.parametrize("reaction_id", BIOMASS_IDS)
 @annotate(title="Growth-associated Maintenance in Biomass Reaction",
-          type="raw", data=dict(), message=dict(), metric=dict())
+          format_type="raw", data=dict(), message=dict(), metric=dict())
 def test_gam_in_biomass(model, reaction_id):
     """
-    Expect the biomass reactions to contain atp and adp.
+    Expect the biomass reactions to contain  ATP and ADP.
 
     The growth-associated maintenance (GAM) term accounts for the energy in
     the form of ATP that is required to synthesize macromolecules such as
@@ -225,8 +230,8 @@ def test_gam_in_biomass(model, reaction_id):
 
 
 @pytest.mark.parametrize("reaction_id", BIOMASS_IDS)
-@annotate(title="Unrealistic Growth Rate In Default Condition", type="raw",
-          data=dict(), message=dict(), metric=dict())
+@annotate(title="Unrealistic Growth Rate In Default Medium",
+          format_type="raw", data=dict(), message=dict(), metric=dict())
 def test_fast_growth_default(model, reaction_id):
     """
     Expect the predicted growth rate for each BOF to be below 10.3972.
@@ -248,10 +253,10 @@ def test_fast_growth_default(model, reaction_id):
 
 @pytest.mark.parametrize("reaction_id", BIOMASS_IDS)
 @annotate(title="Ratio of Direct Metabolites in Biomass Reaction",
-          type="percent", data=dict(), message=dict(), metric=dict())
+          format_type="percent", data=dict(), message=dict(), metric=dict())
 def test_direct_metabolites_in_biomass(model, reaction_id):
     """
-    Expect the biomass reactions to contain atp and adp.
+    Expect the ratio of direct metabolites to be below 0.5.
 
     Some biomass precursors are taken from the media and directly consumed by
     the biomass reaction. It might not be a problem for ions or
@@ -278,24 +283,29 @@ def test_direct_metabolites_in_biomass(model, reaction_id):
     # TODO: Update the threshold as soon as we have an overview of the average!
     ann = test_direct_metabolites_in_biomass.annotation
     reaction = model.reactions.get_by_id(reaction_id)
-    ann["data"][reaction_id] = [
-        m.id for m in biomass.find_direct_metabolites(model, reaction)]
+    try:
+        ann["data"][reaction_id] = get_ids(
+            biomass.find_direct_metabolites(model, reaction))
+    except OptimizationError:
+        ann["data"][reaction_id] = []
+        ann["metric"][reaction_id] = 1.0
+        ann["message"][reaction_id] = "This model does not grow."
+        pytest.skip(ann["message"])
     ann["metric"][reaction_id] = len(ann["data"][reaction_id]) / \
         len(biomass.find_biomass_precursors(model, reaction))
     ann["message"][reaction_id] = wrapper.fill(
-        """{} contains a total of {} direct metabolites ({:.2%}). Specifically
-        these are: {}.""".format(reaction_id,
-                                 len(ann["data"][reaction_id]),
-                                 ann["metric"][reaction_id],
-                                 ann["data"][reaction_id]
-                                 )
-    )
+        """{} contains a total of {} direct metabolites ({:.2%}).
+        Specifically these are: {}.
+        """.format(reaction_id,
+                   len(ann["data"][reaction_id]),
+                   ann["metric"][reaction_id],
+                   ann["data"][reaction_id]))
     assert ann["metric"][reaction_id] < 0.5, ann["message"][reaction_id]
 
 
 @pytest.mark.parametrize("reaction_id", BIOMASS_IDS)
 @annotate(title="Number of Missing Essential Biomass Precursors",
-          type="count", data=dict(), message=dict(), metric=dict())
+          format_type="count", data=dict(), message=dict(), metric=dict())
 def test_essential_precursors_not_in_biomass(model, reaction_id):
     """
     Expect the biomass reaction to contain all essential precursors.
@@ -316,7 +326,7 @@ def test_essential_precursors_not_in_biomass(model, reaction_id):
     RNA: atp, ctp, utp, gtp
     Cofactors: nad, nadp, amet, fad, pydx5p, coa, thmpp, fmn and h2o
 
-    These metabolties were selected based on the results presented by
+    These metabolites were selected based on the results presented by
     DOI:10.1016/j.ymben.2016.12.002
     """
     ann = test_essential_precursors_not_in_biomass.annotation
