@@ -28,7 +28,7 @@ import memote.support.sbo as sbo
 from memote.utils import annotate, truncate, get_ids, wrapper
 
 
-@annotate(title="Metabolites without SBO-Term Annotation", type="count")
+@annotate(title="Metabolite General SBO Presence", format_type="count")
 def test_metabolite_sbo_presence(read_only_model):
     """Expect all metabolites to have a some form of SBO-Term annotation.
 
@@ -36,6 +36,7 @@ def test_metabolite_sbo_presence(read_only_model):
     with terms which indicate the intended function of its individual
     components. The available terms are controlled and relational and can be
     viewed here http://www.ebi.ac.uk/sbo/main/tree.
+
     """
     ann = test_metabolite_sbo_presence.annotation
     ann["data"] = get_ids(sbo.find_components_without_sbo_terms(
@@ -43,8 +44,8 @@ def test_metabolite_sbo_presence(read_only_model):
     try:
         ann["metric"] = len(ann["data"]) / len(read_only_model.metabolites)
         ann["message"] = wrapper.fill(
-            """A total of {} metabolites ({:.2%}) lack annotation with any type of
-            SBO term: {}""".format(
+            """A total of {} metabolites ({:.2%}) lack annotation with any type
+            of SBO term: {}""".format(
                 len(ann["data"]), ann["metric"], truncate(ann["data"])))
     except ZeroDivisionError:
         ann["metric"] = 1.0
@@ -53,7 +54,7 @@ def test_metabolite_sbo_presence(read_only_model):
     assert len(ann["data"]) == len(read_only_model.metabolites), ann["message"]
 
 
-@annotate(title="Reactions without SBO-Term Annotation", type="count")
+@annotate(title="Reaction General SBO Presence", format_type="count")
 def test_reaction_sbo_presence(read_only_model):
     """Expect all reactions to have a some form of SBO-Term annotation.
 
@@ -61,6 +62,7 @@ def test_reaction_sbo_presence(read_only_model):
     with terms which indicate the intended function of its individual
     components. The available terms are controlled and relational and can be
     viewed here http://www.ebi.ac.uk/sbo/main/tree.
+
     """
     ann = test_reaction_sbo_presence.annotation
     ann["data"] = get_ids(sbo.find_components_without_sbo_terms(
@@ -68,8 +70,8 @@ def test_reaction_sbo_presence(read_only_model):
     try:
         ann["metric"] = len(ann["data"]) / len(read_only_model.reactions)
         ann["message"] = wrapper.fill(
-            """A total of {} reactions ({:.2%}) lack annotation with any type of
-            SBO term: {}""".format(
+            """A total of {} reactions ({:.2%}) lack annotation with any type
+            of SBO term: {}""".format(
                 len(ann["data"]), ann["metric"], truncate(ann["data"])))
     except ZeroDivisionError:
         ann["metric"] = 1.0
@@ -78,7 +80,7 @@ def test_reaction_sbo_presence(read_only_model):
     assert len(ann["data"]) == len(read_only_model.reactions), ann["message"]
 
 
-@annotate(title="Genes without SBO-Term Annotation", type="count")
+@annotate(title="Gene General SBO Presence", format_type="count")
 def test_gene_sbo_presence(read_only_model):
     """Expect all genes to have a some form of SBO-Term annotation.
 
@@ -86,6 +88,7 @@ def test_gene_sbo_presence(read_only_model):
     with terms which indicate the intended function of its individual
     components. The available terms are controlled and relational and can be
     viewed here http://www.ebi.ac.uk/sbo/main/tree.
+
     """
     ann = test_gene_sbo_presence.annotation
     ann["data"] = get_ids(sbo.find_components_without_sbo_terms(
@@ -103,7 +106,7 @@ def test_gene_sbo_presence(read_only_model):
     assert len(ann["data"]) == len(read_only_model.genes), ann["message"]
 
 
-@annotate(title="Metabolic Reactions without SBO:0000176", type="count")
+@annotate(title="Metabolic Reaction SBO:0000176 Presence", format_type="count")
 def test_metabolic_reaction_specific_sbo_presence(read_only_model):
     """Expect all metabolic reactions to be annotated with SBO:0000176.
 
@@ -111,6 +114,7 @@ def test_metabolic_reaction_specific_sbo_presence(read_only_model):
     reaction that is not a transport or boundary reaction should be annotated
     with this. The results shown are relative to the total amount of pure
     metabolic reactions.
+
     """
     ann = test_metabolic_reaction_specific_sbo_presence.annotation
     pure = basic.find_pure_metabolic_reactions(read_only_model)
@@ -119,9 +123,9 @@ def test_metabolic_reaction_specific_sbo_presence(read_only_model):
     try:
         ann["metric"] = len(ann["data"]) / len(pure)
         ann["message"] = wrapper.fill(
-            """A total of {} metabolic reactions ({:.2%} of all purely metabolic
-            reactions) lack annotation with the SBO term "SBO:0000176" for
-            'biochemical reaction': {}""".format(
+            """A total of {} metabolic reactions ({:.2%} of all purely
+            metabolic reactions) lack annotation with the SBO term
+            "SBO:0000176" for 'biochemical reaction': {}""".format(
                 len(ann["data"]), ann["metric"], truncate(ann["data"])))
     except ZeroDivisionError:
         ann["metric"] = 1.0
@@ -130,26 +134,32 @@ def test_metabolic_reaction_specific_sbo_presence(read_only_model):
     assert len(ann["data"]) == len(pure), ann["message"]
 
 
-@annotate(title="Transport Reactions without SBO:0000185", type="count")
+@annotate(title="Transport Reaction SBO:0000185 Presence", format_type="count")
 def test_transport_reaction_specific_sbo_presence(read_only_model):
-    """Expect all transport reactions to be annotated with SBO:0000185.
+    """Expect all transport reactions to be annotated properly.
+
+    'SBO:0000185', 'SBO:0000588', 'SBO:0000587', 'SBO:0000655', 'SBO:0000654',
+    'SBO:0000660', 'SBO:0000659', 'SBO:0000657', and 'SBO:0000658' represent
+    the terms 'transport reaction' and 'translocation reaction', in addition
+    to their children (more specific transport reaction labels). Every
+    transport reaction that is not a pure metabolic or boundary reaction should
+    be annotated with one of these terms. The results shown are relative to the
+    total of all transport reactions.
 
-    SBO:0000185 represents the term 'transport reaction'. Every transport
-    reaction that is not a pure metabolic or boundary reaction should be
-    annotated with this. The results shown are relative to the total of all
-    transport reactions.
     """
+    sbo_transport_terms = helpers.TRANSPORT_RXN_SBO_TERMS
     ann = test_transport_reaction_specific_sbo_presence.annotation
     transports = helpers.find_transport_reactions(read_only_model)
     ann["data"] = get_ids(sbo.check_component_for_specific_sbo_term(
-        transports, "SBO:0000185"))
+        transports, sbo_transport_terms))
     try:
         ann["metric"] = len(ann["data"]) / len(transports)
         ann["message"] = wrapper.fill(
             """A total of {} metabolic reactions ({:.2%} of all transport
-            reactions) lack annotation with the SBO term "SBO:0000185" for
+            reactions) lack annotation with one of the SBO terms: {} for
             'biochemical reaction': {}""".format(
-                len(ann["data"]), ann["metric"], truncate(ann["data"])))
+                len(ann["data"]), ann["metric"], sbo_transport_terms,
+                truncate(ann["data"])))
     except ZeroDivisionError:
         ann["metric"] = 1.0
         ann["message"] = "The model has no transport reactions."
@@ -157,12 +167,13 @@ def test_transport_reaction_specific_sbo_presence(read_only_model):
     assert len(ann["data"]) == len(transports), ann["message"]
 
 
-@annotate(title="Metabolites without SBO:0000247", type="count")
+@annotate(title="Metabolite SBO:0000247 Presence", format_type="count")
 def test_metabolite_specific_sbo_presence(read_only_model):
     """Expect all metabolites to be annotated with SBO:0000247.
 
     SBO:0000247 represents the term 'simple chemical'. Every metabolite should
     be annotated with this.
+
     """
     ann = test_metabolite_specific_sbo_presence.annotation
     ann["data"] = get_ids(sbo.check_component_for_specific_sbo_term(
@@ -170,8 +181,8 @@ def test_metabolite_specific_sbo_presence(read_only_model):
     try:
         ann["metric"] = len(ann["data"]) / len(read_only_model.metabolites)
         ann["message"] = wrapper.fill(
-            """A total of {} transport reactions ({:.2%} of all metabolites) lack
-            annotation with the SBO term "SBO:0000247" for
+            """A total of {} transport reactions ({:.2%} of all metabolites)
+            lack annotation with the SBO term "SBO:0000247" for
             'simple chemical': {}""".format(
                 len(ann["data"]), ann["metric"], truncate(ann["data"])))
     except ZeroDivisionError:
@@ -181,12 +192,13 @@ def test_metabolite_specific_sbo_presence(read_only_model):
     assert len(ann["data"]) == len(read_only_model.metabolites), ann["message"]
 
 
-@annotate(title="Genes without SBO:0000243", type="count")
+@annotate(title="Gene SBO:0000243 Presence", format_type="count")
 def test_gene_specific_sbo_presence(read_only_model):
     """Expect all genes to be annotated with SBO:0000243.
 
     SBO:0000243 represents the term 'gene'. Every gene should
     be annotated with this.
+
     """
     ann = test_gene_specific_sbo_presence.annotation
     ann["data"] = get_ids(sbo.check_component_for_specific_sbo_term(
@@ -205,7 +217,7 @@ def test_gene_specific_sbo_presence(read_only_model):
     assert len(ann["data"]) == len(read_only_model.genes), ann["message"]
 
 
-@annotate(title="Exchange reactions without SBO:0000627", type="count")
+@annotate(title="Exchange Reaction SBO:0000627 Presence", format_type="count")
 def test_exchange_specific_sbo_presence(read_only_model):
     """Expect all exchange reactions to be annotated with SBO:0000627.
 
@@ -222,6 +234,7 @@ def test_exchange_specific_sbo_presence(read_only_model):
     this. Exchange reactions differ from demand reactions in that the
     metabolites are removed from or added to the extracellular
     environment only.
+
     """
     ann = test_exchange_specific_sbo_presence.annotation
     exchanges = helpers.find_exchange_rxns(read_only_model)
@@ -230,8 +243,8 @@ def test_exchange_specific_sbo_presence(read_only_model):
     try:
         ann["metric"] = len(ann["data"]) / len(exchanges)
         ann["message"] = wrapper.fill(
-            """A total of {} exchange reactions ({:.2%} of all exchange reactions)
-            lack annotation with the SBO term "SBO:0000627" for
+            """A total of {} exchange reactions ({:.2%} of all exchange
+            reactions) lack annotation with the SBO term "SBO:0000627" for
             'exchange reaction': {}""".format(
                 len(ann["data"]), ann["metric"], truncate(ann["data"])))
     except ZeroDivisionError:
@@ -241,7 +254,7 @@ def test_exchange_specific_sbo_presence(read_only_model):
     assert len(ann["data"]) == len(exchanges), ann["message"]
 
 
-@annotate(title="Demand reactions without SBO:0000628", type="count")
+@annotate(title="Demand Reaction SBO:0000628 Presence", format_type="count")
 def test_demand_specific_sbo_presence(read_only_model):
     """Expect all demand reactions to be annotated with SBO:0000627.
 
@@ -257,6 +270,7 @@ def test_demand_specific_sbo_presence(read_only_model):
     metabolites are not removed from the extracellular environment, but from
     any of the organism's compartments. Demand reactions differ from sink
     reactions in that they are designated as irreversible.
+
     """
     ann = test_demand_specific_sbo_presence.annotation
     demands = helpers.find_demand_reactions(read_only_model)
@@ -276,7 +290,7 @@ def test_demand_specific_sbo_presence(read_only_model):
     assert len(ann["data"]) == len(demands), ann["message"]
 
 
-@annotate(title="Sink reactions without SBO:0000632", type="count")
+@annotate(title="Sink Reactions SBO:0000632 Presence", format_type="count")
 def test_sink_specific_sbo_presence(read_only_model):
     """Expect all sink reactions to be annotated with SBO:0000632.
 
@@ -295,6 +309,7 @@ def test_sink_specific_sbo_presence(read_only_model):
     this. Sink reactions differ from exchange reactions in that the metabolites
     are not removed from the extracellular environment, but from any of the
     organism's compartments.
+
     """
     ann = test_sink_specific_sbo_presence.annotation
     sinks = helpers.find_sink_reactions(read_only_model)
@@ -315,7 +330,7 @@ def test_sink_specific_sbo_presence(read_only_model):
     assert len(ann["data"]) == len(sinks), ann["message"]
 
 
-@annotate(title="Biomass reactions without SBO:0000629", type="count")
+@annotate(title="Biomass Reactions SBO:0000629 Presence", format_type="count")
 def test_biomass_specific_sbo_presence(read_only_model):
     """Expect all biomass reactions to be annotated with SBO:0000629.
 
@@ -334,6 +349,7 @@ def test_biomass_specific_sbo_presence(read_only_model):
     growth.'
     Every reaction representing the biomass production should be annotated with
     this.
+
     """
     ann = test_biomass_specific_sbo_presence.annotation
     biomass = helpers.find_biomass_reaction(read_only_model)
