@@ -20,6 +20,8 @@
 from __future__ import absolute_import
 
 import logging
+from itertools import combinations
+from pylru import lrudecorator
 
 import memote.support.helpers as helpers
 
@@ -46,9 +48,10 @@ def check_gene_protein_reaction_rule_presence(model):
 
 def find_nonzero_constrained_reactions(model):
     """Return list of reactions with non-zero, non-maximal bounds."""
+    lower_bound, upper_bound = helpers.find_bounds(model)
     return [rxn for rxn in model.reactions if
-            0 > rxn.lower_bound > -1000 or
-            0 < rxn.upper_bound < 1000]
+            0 > rxn.lower_bound > lower_bound or
+            0 < rxn.upper_bound < upper_bound]
 
 
 def find_zero_constrained_reactions(model):
@@ -65,9 +68,10 @@ def find_irreversible_reactions(model):
 
 def find_unconstrained_reactions(model):
     """Return list of reactions that are not constrained at all."""
+    lower_bound, upper_bound = helpers.find_bounds(model)
     return [rxn for rxn in model.reactions if
-            rxn.lower_bound <= -1000 and
-            rxn.upper_bound >= 1000]
+            rxn.lower_bound <= lower_bound and
+            rxn.upper_bound >= upper_bound]
 
 
 def find_ngam(model):
@@ -186,13 +190,20 @@ def calculate_metabolic_coverage(model):
 
 def find_protein_complexes(model):
     """
-    Find tuples of gene identifiers that constitute functional enzyme complexes.
+    Find tuples of gene identifiers that comprise functional enzyme complexes.
 
     Parameters
     ----------
     model : cobra.Model
         The metabolic model under investigation.
 
+    Returns
+    -------
+    set
+        A set of tuples of genes that are connected via "AND" in
+        gene-protein-reaction rules (GPR) and thus constitute protein
+        complexes.
+
     """
     protein_complexes = set()
     for rxn in model.reactions:
@@ -204,6 +215,7 @@ def find_protein_complexes(model):
     return protein_complexes
 
 
+@lrudecorator(size=2)
 def find_pure_metabolic_reactions(model):
     """
     Return reactions that are neither transporters, exchanges, nor pseudo.
@@ -214,10 +226,25 @@ def find_pure_metabolic_reactions(model):
         The metabolic model under investigation.
 
     """
-    exchanges = set(model.exchanges)
-    transporters = set(helpers.find_transport_reactions(model))
-    biomass = set(helpers.find_biomass_reaction(model))
-    return set(model.reactions) - (exchanges | transporters | biomass)
+    return set(model.reactions) - helpers.find_interchange_biomass_reactions(
+        model)
+
+
+def is_constrained_reaction(model, rxn):
+    """Return whether a reaction has fixed constraints."""
+    lower_bound, upper_bound = helpers.find_bounds(model)
+    if rxn.reversibility:
+        return rxn.lower_bound > lower_bound or rxn.upper_bound < upper_bound
+    else:
+        return rxn.lower_bound > 0 or rxn.upper_bound < upper_bound
+
+
+def find_oxygen_reactions(model):
+    """Return the directionality of oxygen-producing/-consuming reactions."""
+    o2_in_model = helpers.find_met_in_model(model, "MNXM4")
+    return set([rxn for met in model.metabolites for
+                rxn in met.reactions if met.formula == "O2" or
+                met in o2_in_model])
 
 
 def find_unique_metabolites(model):
@@ -226,7 +253,84 @@ def find_unique_metabolites(model):
     return set(met.id.split("_", 1)[0] for met in model.metabolites)
 
 
+def find_duplicate_metabolites_in_compartments(model):
+    """
+    Return list of metabolites with duplicates in the same compartment.
+
+    This function identifies duplicate metabolites in each compartment by
+    determining if any two metabolites have identical InChI-key annotations.
+    For instance, this function would find compounds with IDs ATP1 and ATP2 in
+    the cytosolic compartment, with both having the same InChI annotations.
+
+    Parameters
+    ----------
+    model : cobra.Model
+        The metabolic model under investigation.
+
+    Returns
+    -------
+    list
+        A list of tuples of duplicate metabolites.
+
+    """
+    duplicates = []
+    for compartment in model.compartments:
+        ann_mets = [(met, met.annotation) for met in model.metabolites
+                    if met.compartment == compartment and
+                    "inchikey" in met.annotation]
+        for a, b in combinations(ann_mets, 2):
+            if a[1]["inchikey"] == b[1]["inchikey"]:
+                duplicates.append((a[0], b[0]))
+    return duplicates
+
+
+def find_duplicate_reactions(model):
+    """
+    Return list of reactions with duplicates.
+
+    This function identifies duplicate reactions in each compartment by
+    determining if any two metabolites have identical annotations.
+    For instance, this function would find compounds with IDs ATP1 and ATP2 in
+    the cytosolic compartment, with both having the same InChI annotations.
+
+    Parameters
+    ----------
+    model : cobra.Model
+        The metabolic model under investigation.
+
+    Returns
+    -------
+    list
+        A list of tuples of duplicate reacions.
+
+    """
+    duplicates = []
+    rxn_db_identifiers = ["metanetx.reaction", "kegg.reaction", "brenda",
+                          "rhea", "biocyc", "bigg.reaction"]
+
+    ann_rxns = []
+    for rxn in model.reactions:
+        ann = []
+        for key in rxn_db_identifiers:
+            if key in rxn.annotation:
+                if type(rxn.annotation[key]) is list:
+                    ann.extend([(key, elem) for elem in rxn.annotation[key]])
+                else:
+                    ann.append((key, rxn.annotation[key]))
+        ann_rxns.append((rxn, frozenset(ann)))
+    for (rxn_a, ann_a), (rxn_b, ann_b) in combinations(ann_rxns, 2):
+        if len(ann_a & ann_b) > 0:
+            duplicates.append((rxn_a.id, rxn_b.id))
+    return duplicates
+
+
 def check_transport_reaction_gpr_presence(model):
     """Return the list of transport reactions that have no associated gpr."""
     return [rxn for rxn in helpers.find_transport_reactions(model)
             if not rxn.gene_reaction_rule]
+
+
+def find_medium_metabolites(model):
+    """Return the list of metabolites ingested/excreted by the model."""
+    return [met.id for rxn in model.medium
+            for met in model.reactions.get_by_id(rxn).metabolites]
