@@ -26,6 +26,8 @@ import numpy as np
 from cobra import Reaction
 from cobra.exceptions import Infeasible
 from cobra.flux_analysis import flux_variability_analysis
+from optlang.interface import OPTIMAL, INFEASIBLE
+from optlang.symbolics import Zero
 
 import memote.support.consistency_helpers as con_helpers
 import memote.support.helpers as helpers
@@ -59,7 +61,7 @@ ENERGY_COUPLES = {
 
 def check_stoichiometric_consistency(model):
     """
-    Verify the consistency of the model stoichiometry.
+    Verify the consistency of the model's stoichiometry.
 
     Parameters
     ----------
@@ -71,33 +73,34 @@ def check_stoichiometric_consistency(model):
     See [1]_ section 3.1 for a complete description of the algorithm.
 
     .. [1] Gevorgyan, A., M. G Poolman, and D. A Fell.
-           "Detection of Stoichiometric Inconsistencies in Biomolecular Models."
+           "Detection of Stoichiometric Inconsistencies in Biomolecular
+           Models."
            Bioinformatics 24, no. 19 (2008): 2245.
 
     """
-    Model, Constraint, Variable, Objective = con_helpers.get_interface(model)
+    problem = model.problem
     # The transpose of the stoichiometric matrix N.T in the paper.
-    stoich_trans = Model()
+    stoich_trans = problem.Model()
     internal_rxns = con_helpers.get_internals(model)
     metabolites = set(met for rxn in internal_rxns for met in rxn.metabolites)
     LOGGER.info("model '%s' has %d internal reactions", model.id,
                 len(internal_rxns))
     LOGGER.info("model '%s' has %d internal metabolites", model.id,
                 len(metabolites))
-    for metabolite in metabolites:
-        stoich_trans.add(Variable(metabolite.id, lb=1))
+
+    stoich_trans.add([problem.Variable(m.id, lb=1) for m in metabolites])
     stoich_trans.update()
     con_helpers.add_reaction_constraints(
-        stoich_trans, internal_rxns, Constraint)
+        stoich_trans, internal_rxns, problem.Constraint)
     # The objective is to minimize the metabolite mass vector.
-    stoich_trans.objective = Objective(1)
+    stoich_trans.objective = problem.Objective(
+        Zero, direction="min", sloppy=True)
     stoich_trans.objective.set_linear_coefficients(
         {var: 1. for var in stoich_trans.variables})
-    stoich_trans.objective.direction = "min"
     status = stoich_trans.optimize()
-    if status == "optimal":
+    if status == OPTIMAL:
         return True
-    elif status == "infeasible":
+    elif status == INFEASIBLE:
         return False
     else:
         raise RuntimeError(
@@ -121,46 +124,48 @@ def find_unconserved_metabolites(model):
 
 
     .. [1] Gevorgyan, A., M. G Poolman, and D. A Fell.
-           "Detection of Stoichiometric Inconsistencies in Biomolecular Models."
+           "Detection of Stoichiometric Inconsistencies in Biomolecular
+           Models."
            Bioinformatics 24, no. 19 (2008): 2245.
 
     """
-    Model, Constraint, Variable, Objective = con_helpers.get_interface(model)
-    stoich_trans = Model()
+    problem = model.problem
+    stoich_trans = problem.Model()
     internal_rxns = con_helpers.get_internals(model)
     metabolites = set(met for rxn in internal_rxns for met in rxn.metabolites)
     # The binary variables k[i] in the paper.
     k_vars = list()
     for met in metabolites:
         # The element m[i] of the mass vector.
-        m_var = Variable(met.id)
-        k_var = Variable("k_{}".format(met.id), type="binary")
+        m_var = problem.Variable(met.id)
+        k_var = problem.Variable("k_{}".format(met.id), type="binary")
         k_vars.append(k_var)
         stoich_trans.add([m_var, k_var])
         # This constraint is equivalent to 0 <= k[i] <= m[i].
-        stoich_trans.add(Constraint(
+        stoich_trans.add(problem.Constraint(
             k_var - m_var, ub=0, name="switch_{}".format(met.id)))
     stoich_trans.update()
     con_helpers.add_reaction_constraints(
-        stoich_trans, internal_rxns, Constraint)
+        stoich_trans, internal_rxns, problem.Constraint)
     # The objective is to maximize the binary indicators k[i], subject to the
     # above inequality constraints.
-    stoich_trans.objective = Objective(1)
+    stoich_trans.objective = problem.Objective(
+        Zero, sloppy=True, direction="max")
     stoich_trans.objective.set_linear_coefficients(
         {var: 1. for var in k_vars})
-    stoich_trans.objective.direction = "max"
     status = stoich_trans.optimize()
-    if status == "optimal":
+    if status == OPTIMAL:
         # TODO: See if that could be a Boolean test `bool(var.primal)`.
         return set([model.metabolites.get_by_id(var.name[2:])
                     for var in k_vars if var.primal < 0.8])
     else:
         raise RuntimeError(
             "Could not compute list of unconserved metabolites."
-            " Solver status is '{}'"
-            " (only optimal or infeasible expected).".format(status))
+            " Solver status is '{}' (only optimal expected).".format(status))
 
 
+# FIXME: The results of this function are currently inconsistent with
+# published results.
 def find_inconsistent_min_stoichiometry(model, atol=1e-13):
     """
     Detect inconsistent minimal net stoichiometries.
@@ -177,9 +182,11 @@ def find_inconsistent_min_stoichiometry(model, atol=1e-13):
     -----
     See [1]_ section 3.3 for a complete description of the algorithm.
 
-
+    References
+    ----------
     .. [1] Gevorgyan, A., M. G Poolman, and D. A Fell.
-           "Detection of Stoichiometric Inconsistencies in Biomolecular Models."
+           "Detection of Stoichiometric Inconsistencies in Biomolecular
+           Models."
            Bioinformatics 24, no. 19 (2008): 2245.
 
     """
@@ -189,7 +196,8 @@ def find_inconsistent_min_stoichiometry(model, atol=1e-13):
     unconserved_mets = find_unconserved_metabolites(model)
     LOGGER.info("model has %d unconserved metabolites", len(unconserved_mets))
     internal_rxns = con_helpers.get_internals(model)
-    internal_mets = set(met for rxn in internal_rxns for met in rxn.metabolites)
+    internal_mets = set(
+        met for rxn in internal_rxns for met in rxn.metabolites)
     get_id = attrgetter("id")
     reactions = sorted(internal_rxns, key=get_id)
     metabolites = sorted(internal_mets, key=get_id)
@@ -373,7 +381,7 @@ def find_mass_unbalanced_reactions(reactions):
     Parameters
     ----------
     reactions : iterable
-        An iterable of ``cobra.Reaction``s.
+        An iterable of cobra.Reaction's.
 
     """
     return [
@@ -387,7 +395,7 @@ def find_charge_unbalanced_reactions(reactions):
     Parameters
     ----------
     reactions : iterable
-        An iterable of ``cobra.Reaction``s.
+        An iterable of cobra.Reaction's.
 
     """
     return [
@@ -544,6 +552,58 @@ def find_metabolites_consumed_with_closed_bounds(model):
     return mets_consumed
 
 
+def find_metabolites_not_produced_with_open_bounds(model):
+    """
+    Return metabolites that cannot be produced with open boundary reactions.
+
+    Inverse case from 'find_metabolites_produced_with_closed_bounds', just
+    like metabolites should not be produced from nothing, metabolites should
+    all be produced with open exhanges.
+
+    Parameters
+    ----------
+    model : cobra.Model
+        The metabolic model under investigation.
+
+    """
+    mets_not_produced = list()
+    helpers.open_boundaries(model)
+    for met in model.metabolites:
+        with model:
+            exch = model.add_boundary(
+                met, type="irrex", reaction_id="IRREX", lb=0, ub=1)
+            solution = helpers.run_fba(model, exch.id)
+            if solution is np.nan or solution == 0:
+                mets_not_produced.append(met)
+    return mets_not_produced
+
+
+def find_metabolites_not_consumed_with_open_bounds(model):
+    """
+    Return metabolites that cannot be consumed with open boundary reactions.
+
+    Reverse case from 'find_metabolites_not_produced_with_open_bounds', just
+    like metabolites should all be produced with open exchanges, they should
+    also all be consumed with open exchanges.
+
+    Parameters
+    ----------
+    model : cobra.Model
+        The metabolic model under investigation.
+
+    """
+    mets_not_consumed = list()
+    helpers.open_boundaries(model)
+    for met in model.metabolites:
+        with model:
+            exch = model.add_boundary(
+                met, type="irrex", reaction_id="IRREX", lb=-1, ub=0)
+            solution = helpers.run_fba(model, exch.id, direction="min")
+            if solution is np.nan or solution == 0:
+                mets_not_consumed.append(met)
+    return mets_not_consumed
+
+
 def find_reactions_with_unbounded_flux_default_condition(model):
     """
     Return list of reactions whose flux is unbounded in the default condition.
@@ -553,8 +613,8 @@ def find_reactions_with_unbounded_flux_default_condition(model):
     model : cobra.Model
         The metabolic model under investigation.
 
-    Returns:
-    --------
+    Returns
+    -------
     unlimited_reactions: list
         A list of reactions that in default modeling conditions are able to
         carry flux as high/low as the systems maximal and minimal bounds.
