@@ -21,21 +21,27 @@ from __future__ import absolute_import
 
 import logging
 import re
-from builtins import dict
 from collections import defaultdict
-from cobra.exceptions import Infeasible
+from operator import itemgetter
+
 
 import numpy as np
 import pandas as pd
 from six import iteritems, itervalues
 from sympy import expand
 from importlib_resources import open_text
+from cobra.exceptions import Infeasible
+from pylru import lrudecorator
 
 import memote.utils as utils
 import memote.support.data
 
 LOGGER = logging.getLogger(__name__)
 
+TRANSPORT_RXN_SBO_TERMS = ['SBO:0000185', 'SBO:0000588', 'SBO:0000587',
+                           'SBO:0000655', 'SBO:0000654', 'SBO:0000660',
+                           'SBO:0000659', 'SBO:0000657', 'SBO:0000658']
+
 
 # Read the MetaNetX shortlist to identify specific metabolite IDs across
 # different namespaces.
@@ -44,7 +50,7 @@ with open_text(memote.support.data, "met_id_shortlist.json",
     METANETX_SHORTLIST = pd.read_json(file_handle)
 
 
-# Provide a compartment shortlist to identify specfic compartments whenever
+# Provide a compartment shortlist to identify specific compartments whenever
 # necessary.
 COMPARTMENT_SHORTLIST = {
     'ce': ['cell envelope'],
@@ -114,6 +120,7 @@ def find_transported_elements(rxn):
     return delta_dict
 
 
+@lrudecorator(size=2)
 def find_transport_reactions(model):
     """
     Return a list of all transport reactions.
@@ -128,40 +135,110 @@ def find_transport_reactions(model):
     A transport reaction is defined as follows:
     1. It contains metabolites from at least 2 compartments and
     2. at least 1 metabolite undergoes no chemical reaction, i.e.,
-    the formula stays the same on both sides of the equation.
+    the formula and/or annotation stays the same on both sides of the equation.
 
-    This function will not identify transport via the PTS System.
+    A notable exception is transport via PTS, which also contains the following
+    restriction:
+    3. The transported metabolite(s) are transported into a compartment through
+    the exchange of a phosphate group.
+
+    An example of transport via PTS would be
+    pep(c) + glucose(e) -> glucose-6-phosphate(c) + pyr(c)
+
+    Reactions similar to transport via PTS (referred to as "modified transport
+    reactions") follow a similar pattern:
+    A(x) + B-R(y) -> A-R(y) + B(y)
+
+    Such modified transport reactions can be detected, but only when a formula
+    field exists for all metabolites in a particular reaction. If this is not
+    the case, transport reactions are identified through annotations, which
+    cannot detect modified transport reactions.
 
     """
     transport_reactions = []
-    for rxn in model.reactions:
-        # Collecting criteria to classify transporters by.
-        rxn_reactants = set([met.formula for met in rxn.reactants])
-        rxn_products = set([met.formula for met in rxn.products])
-        # Looking for formulas that stay the same on both side of the reaction.
-        transported_mets = \
-            [formula for formula in rxn_reactants if formula in rxn_products]
-        # Collect information on the elemental differences between
-        # compartments in the reaction.
-        delta_dicts = find_transported_elements(rxn)
-        non_zero_array = [v for (k, v) in iteritems(delta_dicts) if v != 0]
-        # Weeding out reactions such as oxidoreductases where no net
-        # transport of Hydrogen is occurring, but rather just an exchange of
-        # electrons or charges effecting a change in protonation.
-        if set(transported_mets) != set('H') and list(
-            delta_dicts.keys()
-        ) == ['H']:
-            pass
-        # All other reactions for which the amount of transported elements is
-        # not zero, which are not part of the model's exchange nor
-        # biomass reactions, are defined as transport reactions.
-        # This includes reactions where the transported metabolite reacts with
-        # a carrier molecule.
-        elif sum(non_zero_array) and rxn not in model.exchanges and \
-                rxn not in find_biomass_reaction(model):
+    transport_rxn_candidates = set(model.reactions) - set(model.exchanges) \
+        - set(find_biomass_reaction(model))
+    # Add all labeled transport reactions
+    sbo_matches = set([rxn for rxn in transport_rxn_candidates if
+                       rxn.annotation is not None and
+                       'SBO' in rxn.annotation and
+                       rxn.annotation['SBO'] in TRANSPORT_RXN_SBO_TERMS])
+    if len(sbo_matches) > 0:
+        transport_reactions += list(sbo_matches)
+    # Find unlabeled transport reactions via formula or annotation checks
+    for rxn in transport_rxn_candidates:
+        # Check if metabolites have formula field
+        rxn_mets = set([met.formula for met in rxn.metabolites])
+        if (None not in rxn_mets) and (len(rxn_mets) != 0):
+            if is_transport_reaction_formulae(rxn):
+                transport_reactions.append(rxn)
+        if is_transport_reaction_annotations(rxn):
             transport_reactions.append(rxn)
 
-    return transport_reactions
+    return set(transport_reactions)
+
+
+def is_transport_reaction_formulae(rxn):
+    """
+    Return boolean if a reaction is a transport reaction (from formulae).
+
+    Parameters
+    ----------
+    rxn: cobra.Reaction
+        The metabolic reaction under investigation.
+
+    """
+    # Collecting criteria to classify transporters by.
+    rxn_reactants = set([met.formula for met in rxn.reactants])
+    rxn_products = set([met.formula for met in rxn.products])
+    # Looking for formulas that stay the same on both side of the reaction.
+    transported_mets = \
+        [formula for formula in rxn_reactants if formula in rxn_products]
+    # Collect information on the elemental differences between
+    # compartments in the reaction.
+    delta_dicts = find_transported_elements(rxn)
+    non_zero_array = [v for (k, v) in iteritems(delta_dicts) if v != 0]
+    # Excluding reactions such as oxidoreductases where no net
+    # transport of Hydrogen is occurring, but rather just an exchange of
+    # electrons or charges effecting a change in protonation.
+    if set(transported_mets) != set('H') and list(
+        delta_dicts.keys()
+    ) == ['H']:
+        pass
+    # All other reactions for which the amount of transported elements is
+    # not zero, which are not part of the model's exchange nor
+    # biomass reactions, are defined as transport reactions.
+    # This includes reactions where the transported metabolite reacts with
+    # a carrier molecule.
+    elif sum(non_zero_array):
+        return True
+
+
+def is_transport_reaction_annotations(rxn):
+    """
+    Return boolean if a reaction is a transport reaction (from annotations).
+
+    Parameters
+    ----------
+    rxn: cobra.Reaction
+        The metabolic reaction under investigation.
+
+    """
+    reactants = set([(k, tuple(v)) for met in rxn.reactants
+                     for k, v in iteritems(met.annotation)
+                     if met.id is not "H" and k is not None and v is not None])
+    products = set([(k, tuple(v)) for met in rxn.products
+                    for k, v in iteritems(met.annotation)
+                    if met.id is not "H" and k is not None and v is not None])
+    # Find intersection between reactant annotations and
+    # product annotations to find common metabolites between them,
+    # satisfying the requirements for a transport reaction. Reactions such
+    # as those involving oxidoreductases (where no net transport of
+    # Hydrogen is occurring, but rather just an exchange of electrons or
+    # charges effecting a change in protonation) are excluded.
+    transported_mets = reactants & products
+    if len(transported_mets) > 0:
+        return True
 
 
 def find_converting_reactions(model, pair):
@@ -186,6 +263,7 @@ def find_converting_reactions(model, pair):
     second = set(find_met_in_model(model, pair[1]))
     hits = list()
     for rxn in model.reactions:
+        # FIXME: Use `set.issubset` much more idiomatic.
         if len(first & set(rxn.reactants)) > 0 and len(
                 second & set(rxn.products)) > 0:
             hits.append(rxn)
@@ -195,10 +273,22 @@ def find_converting_reactions(model, pair):
     return frozenset(hits)
 
 
+@lrudecorator(size=2)
 def find_biomass_reaction(model):
     """
     Return a list of the biomass reaction(s) of the model.
 
+    This function identifies possible biomass reactions using two steps:
+    1. Return reactions that include the SBO annotation "SBO:0000629" for
+    biomass.
+    If no reactions can be identifies this way:
+    2. Look for the ``buzzwords`` "biomass", "growth" and "bof" in reaction IDs.
+    3. Look for metabolite IDs or names that contain the ``buzzword`` "biomass"
+    and obtain the set of reactions they are involved in.
+    4. Remove boundary reactions from this set.
+    5. Return the union of reactions that match the buzzwords and of the
+    reactions that metabolites are involved in that match the buzzword.
+
     Parameters
     ----------
     model : cobra.Model
@@ -206,7 +296,8 @@ def find_biomass_reaction(model):
 
     Returns
     -------
-    list of biomass reactions
+    list
+        Identified biomass reactions.
 
     """
     sbo_matches = set([rxn for rxn in model.reactions if
@@ -238,13 +329,7 @@ def find_biomass_reaction(model):
     return list(buzzword_matches | biomass_met_matches)
 
 
-def df2dict(df):
-    """Turn a `pandas.DataFrame` into a `dict` of lists."""
-    blob = dict((key, df[key].tolist()) for key in df.columns)
-    blob["index"] = df.index.tolist()
-    return blob
-
-
+@lrudecorator(size=2)
 def find_demand_reactions(model):
     u"""
     Return a list of demand reactions.
@@ -252,7 +337,7 @@ def find_demand_reactions(model):
     Parameters
     ----------
     model : cobra.Model
-        A cobrapy metabolic model
+        The metabolic model under investigation.
 
     Notes
     -----
@@ -287,6 +372,7 @@ def find_demand_reactions(model):
             extracellular not in rxn.get_compartments()]
 
 
+@lrudecorator(size=2)
 def find_sink_reactions(model):
     u"""
     Return a list of sink reactions.
@@ -294,7 +380,7 @@ def find_sink_reactions(model):
     Parameters
     ----------
     model : cobra.Model
-        A cobrapy metabolic model
+        The metabolic model under investigation.
 
     Notes
     -----
@@ -328,6 +414,7 @@ def find_sink_reactions(model):
             extracellular not in rxn.get_compartments()]
 
 
+@lrudecorator(size=2)
 def find_exchange_rxns(model):
     u"""
     Return a list of exchange reactions.
@@ -335,7 +422,7 @@ def find_exchange_rxns(model):
     Parameters
     ----------
     model : cobra.Model
-        A cobrapy metabolic model
+        The metabolic model under investigation.
 
     Notes
     -----
@@ -367,6 +454,30 @@ def find_exchange_rxns(model):
             if extracellular in rxn.get_compartments()]
 
 
+def find_interchange_biomass_reactions(model, biomass=None):
+    """
+    Return the set of all transport, boundary, and biomass reactions.
+
+    These reactions are either pseudo-reactions, or incorporated to allow
+    metabolites to pass between compartments. Some tests focus on purely
+    metabolic reactions and hence exclude this set.
+
+    Parameters
+    ----------
+    model : cobra.Model
+        The metabolic model under investigation.
+    biomass : list or cobra.Reaction, optional
+        A list of cobrapy biomass reactions.
+
+    """
+    # exchanges in this case also refer to sink and demand reactions.
+    exchanges = set(model.exchanges)
+    transporters = find_transport_reactions(model)
+    if biomass is None:
+        biomass = set(find_biomass_reaction(model))
+    return exchanges | transporters | biomass
+
+
 def find_functional_units(gpr_str):
     """
     Return an iterator of gene IDs grouped by boolean rules from the gpr_str.
@@ -380,8 +491,8 @@ def find_functional_units(gpr_str):
     Parameters
     ----------
     gpr_str : string
-            A string consisting of gene ids and the boolean expressions 'and'
-            and 'or'
+        A string consisting of gene ids and the boolean expressions 'and'
+        and 'or'.
 
     """
     algebraic_form = re.sub('[Oo]r', '+', re.sub('[Aa]nd', '*', gpr_str))
@@ -397,12 +508,12 @@ def run_fba(model, rxn_id, direction="max", single_value=True):
     Parameters
     ----------
     model : cobra.Model
-        A cobrapy metabolic model
+        The metabolic model under investigation.
     rxn_id : string
-        A string containing the reaction ID of the desired FBA objective
+        A string containing the reaction ID of the desired FBA objective.
     direction: string
         A string containing either "max" or "min" to specify the direction
-        of the desired FBA objective function
+        of the desired FBA objective function.
     single_value: boolean
         Indicates whether the results for all reactions are gathered from the
         solver, or only the result for the objective value.
@@ -410,7 +521,7 @@ def run_fba(model, rxn_id, direction="max", single_value=True):
     Returns
     -------
     cobra.solution
-        The cobra solution object for the corresponding FBA problem
+        The cobra solution object for the corresponding FBA problem.
 
     """
     model.objective = model.reactions.get_by_id(rxn_id)
@@ -440,7 +551,7 @@ def close_boundaries_sensibly(model):
     Parameters
     ----------
     model : cobra.Model
-        A cobrapy metabolic model
+        The metabolic model under investigation.
 
     Returns
     -------
@@ -464,6 +575,11 @@ def open_boundaries(model):
 
     Parameters
     ----------
+    model : cobra.Model
+        The metabolic model under investigation.
+
+    Returns
+    ----------
     cobra.Model
         A cobra model with all boundary reactions opened.
 
@@ -479,7 +595,7 @@ def metabolites_per_compartment(model, compartment_id):
     Parameters
     ----------
     model : cobra.Model
-        A cobrapy metabolic model
+        The metabolic model under investigation.
     compartment_id : string
         Model specific compartment identifier.
 
@@ -489,9 +605,8 @@ def metabolites_per_compartment(model, compartment_id):
         List of metabolites belonging to a given compartment.
 
     """
-    return [met for
-            met in model.metabolites if
-            met.compartment == compartment_id]
+    return [met for met in model.metabolites
+            if met.compartment == compartment_id]
 
 
 def largest_compartment_id_met(model):
@@ -501,7 +616,7 @@ def largest_compartment_id_met(model):
     Parameters
     ----------
     model : cobra.Model
-        A cobrapy metabolic model
+        The metabolic model under investigation.
 
     Returns
     -------
@@ -509,19 +624,17 @@ def largest_compartment_id_met(model):
         Compartment ID of the compartment with the most metabolites.
 
     """
-    size = {compartment_id: len(metabolites_per_compartment(
-                                model, compartment_id))
-            for compartment_id in model.compartments.keys()}
-    candidate = sorted(size, key=size.get, reverse=True)[0]
-    tie = size.copy()
-    tie.pop(candidate)
-    cand_tie = sorted(tie, key=tie.get, reverse=True)[0]
-    if size[candidate] not in tie.values():
-        return candidate
-    else:
+    # Sort compartments by decreasing size and extract the largest two.
+    candidate, second = sorted(
+        ((c, len(metabolites_per_compartment(model, c)))
+         for c in model.compartments), reverse=True, key=itemgetter(1))[:2]
+    # Compare the size of the compartments.
+    if candidate[1] == second[1]:
         raise RuntimeError("There is a tie for the largest compartment. "
                            "Compartment {} and {} have equal amounts of "
-                           "metabolites.".format(candidate, cand_tie))
+                           "metabolites.".format(candidate[0], second[0]))
+    else:
+        return candidate[0]
 
 
 def find_compartment_id_in_model(model, compartment_id):
@@ -531,7 +644,7 @@ def find_compartment_id_in_model(model, compartment_id):
     Parameters
     ----------
     model : cobra.Model
-        A cobrapy metabolic model
+        The metabolic model under investigation.
     compartment_id : string
         Memote internal compartment identifier used to access compartment name
         shortlist to look up potential compartment names.
@@ -539,7 +652,7 @@ def find_compartment_id_in_model(model, compartment_id):
     Returns
     -------
     string
-        Compartment identifier in the model corresponding to compartment_id
+        Compartment identifier in the model corresponding to compartment_id.
 
     """
     if compartment_id not in COMPARTMENT_SHORTLIST.keys():
@@ -569,12 +682,12 @@ def find_compartment_id_in_model(model, compartment_id):
 
 def find_met_in_model(model, mnx_id, compartment_id=None):
     """
-    Identify metabolites in the model by looking up IDs in METANETX_SHORTLIST.
+    Return specific metabolites by looking up IDs in METANETX_SHORTLIST.
 
     Parameters
     ----------
     model : cobra.Model
-        A cobrapy metabolic model
+        The metabolic model under investigation.
     mnx_id : string
         Memote internal MetaNetX metabolite identifier used to map between
         cross-references in the METANETX_SHORTLIST.
@@ -584,7 +697,8 @@ def find_met_in_model(model, mnx_id, compartment_id=None):
 
     Returns
     -------
-    list of cobra.Metabolite
+    list
+        cobra.Metabolite(s) matching the mnx_id.
 
     """
     if mnx_id not in METANETX_SHORTLIST.columns:
@@ -626,7 +740,7 @@ def find_met_in_model(model, mnx_id, compartment_id=None):
                            "a single metabolite in compartment {} that "
                            "corresponds to the following MetaNetX "
                            "identifier: {}."
-                           "Instead these candidates where found: {}."
+                           "Instead these candidates were found: {}."
                            "Check that metabolite compartment tags are "
                            "correct. Consider switching to a namespace scheme "
                            "where identifiers are truly "
@@ -638,3 +752,46 @@ def find_met_in_model(model, mnx_id, compartment_id=None):
                            )
     else:
         return candidates_in_compartment
+
+
+def find_objective_function(model):
+    """
+    Return reactions that are part of the objective function.
+
+    Parameters
+    ----------
+    model : cobra.Model
+        The metabolic model under investigation.
+
+    """
+    return [rxn for rxn in model.reactions if rxn.objective_coefficient != 0]
+
+
+@lrudecorator(size=2)
+def find_bounds(model):
+    """
+    Return the maximal and minimal bounds of the reactions of the model.
+
+    Bounds can vary from model to model. Cobrapy defaults to (-1000, 1000) but
+    this may not be the case for merged or autogenerated models. In these
+    cases, this function is used to iterate over all the bounds of all the
+    reactions and find the most extreme bound values in the model, which are
+    then used as its default max and min bound values.
+
+    Parameters
+    ----------
+    model : cobra.Model
+        The metabolic model under investigation.
+
+    """
+    try:
+        lower_bound = min(rxn.lower_bound for rxn in model.reactions)
+        upper_bound = max(rxn.upper_bound for rxn in model.reactions)
+    except ValueError:
+        lower_bound = -1000
+        upper_bound = 1000
+
+    if lower_bound != 0 or upper_bound != 0:
+        return lower_bound, upper_bound
+    else:
+        return -1000, 1000
