@@ -19,13 +19,18 @@
 
 from __future__ import absolute_import
 
+import json
 import logging
 from builtins import dict, str
+
+from future.utils import raise_with_traceback
 from numpydoc.docscrape import NumpyDocString
 from textwrap import TextWrapper
+from depinfo import print_dependencies
 
-__all__ = ("register_with", "annotate", "get_ids", "truncate",
-           "log_json_incompatible_types")
+__all__ = ("register_with", "annotate", "get_ids",
+           "get_ids_and_bounds", "truncate", "wrapper",
+           "log_json_incompatible_types", "show_versions", "jsonify")
 
 LOGGER = logging.getLogger(__name__)
 
@@ -46,10 +51,12 @@ def register_with(registry):
 
     Examples
     --------
-    REGISTRY = dict()
-    @register_with(REGISTRY)
-    def build_empty(base):
-        return base
+    .. code-block:: python
+
+        REGISTRY = dict()
+        @register_with(REGISTRY)
+        def build_empty(base):
+            return base
 
     """
     def decorator(func):
@@ -58,29 +65,29 @@ def register_with(registry):
     return decorator
 
 
-# TODO: Change naming of the 'type' argument once the angular app is completed.
-# It is misleading.
-def annotate(title, type, message=None, data=None, metric=1.0):
+def annotate(title, format_type, message=None, data=None, metric=1.0):
     """
-    Annotate a test case.
+    Annotate a test case with info that should be displayed in the reports.
 
     Parameters
     ----------
     title : str
         A human-readable descriptive title of the test case.
-    type : str
-        A sting that determines how the result data is formatted in the report.
-        It is expected not to be None.
-        - 'number' : 'data' is a single number which can be an integer or
+    format_type : str
+        A string that determines how the result data is formatted in the
+        report. It is expected not to be None.
+
+        * 'number' : 'data' is a single number which can be an integer or
           float and should be represented as such.
-        - 'count' : 'data' is a list, set or tuple. Choosing 'count' will
+        * 'count' : 'data' is a list, set or tuple. Choosing 'count' will
           display the length of that list e.g. number of metabolites without
           formula.
-        - 'percent' : Instead of 'data' the content of 'metric' ought to be
+        * 'percent' : Instead of 'data' the content of 'metric' ought to be
           displayed e.g. percentage of metabolites without charge.
           'metric' is expected to be a floating point number.
-        - 'raw' : 'data' is ought to be displayed "as is" without formatting.
+        * 'raw' : 'data' is ought to be displayed "as is" without formatting.
           This option is appropriate for single strings or a boolean output.
+
     message : str
         A short written explanation that states and possibly explains the test
         result.
@@ -104,7 +111,7 @@ def annotate(title, type, message=None, data=None, metric=1.0):
     predefined keys as a dictionary.
 
     """
-    if type not in TYPES:
+    if format_type not in TYPES:
         raise ValueError(
             "Invalid type. Expected one of: {}.".format(", ".join(TYPES)))
 
@@ -114,7 +121,7 @@ def annotate(title, type, message=None, data=None, metric=1.0):
             summary=extended_summary(func),
             message=message,
             data=data,
-            type=type,
+            format_type=format_type,
             metric=metric)
         return func
     return decorator
@@ -125,6 +132,12 @@ def get_ids(iterable):
     return [element.id for element in iterable]
 
 
+def get_ids_and_bounds(iterable):
+    """Retrieve the identifier and bounds of a  number of objects."""
+    return ["{0.lower_bound} <= {0.id} <= {0.upper_bound}".format(elem) for
+            elem in iterable]
+
+
 def truncate(sequence):
     """
     Create a potentially shortened text display of a list.
@@ -179,7 +192,7 @@ def extended_summary(func):
     Parameters
     ----------
     func : function
-        A scored or unscored test function used in `memote report snapshot`
+        A test function used in `memote report snapshot`
 
     Returns
     -------
@@ -189,3 +202,37 @@ def extended_summary(func):
     """
     doc = NumpyDocString(func.__doc__)
     return "\n".join(doc["Extended Summary"])
+
+
+def show_versions():
+    """Print formatted dependency information to stdout."""
+    print_dependencies("memote")
+
+
+def jsonify(obj, pretty=False):
+    """
+    Turn a nested object into a (compressed) JSON string.
+
+    Parameters
+    ----------
+    obj : dict
+        Any kind of dictionary structure.
+    pretty : bool, optional
+        Whether to format the resulting JSON in a more legible way (
+        default False).
+
+    """
+    # TODO: Use compression of JSON in future.
+    #     results=b64encode(compress(
+    #         json.dumps(self.data).encode("UTF-16"), level=9)))
+    if pretty:
+        params = dict(sort_keys=True, indent=2,
+                      separators=(",", ": "), ensure_ascii=False)
+    else:
+        params = dict(sort_keys=False, indent=None,
+                      separators=(",", ":"), ensure_ascii=False)
+    try:
+        return json.dumps(obj, **params)
+    except TypeError as error:
+        log_json_incompatible_types(obj)
+        raise_with_traceback(error)
